//  
//  The original FieldMeter class is Copyright (c) 1994 by Mike Romberg 
//    ( romberg@fsl.noaa.gov )
//
//  Modifications from FieldMeter class done in Oct. 1995
//    by Brian Grayson ( bgrayson@netbsd.org )
//
//  Modifications from FieldMeterDecay class done in Oct. 1998
//    by Scott McNab ( jedi@tartarus.uwa.edu.au )
//
// $Id$
//

// In order to use the FieldMeterGraph class in place of a FieldMeter class in
// a meter file (say, cpumeter.cc), make the following changes:
//   1.  Change cpumeter.h to include fieldmetergraph.h instead of
//       fieldmeter.h
//   2.  Change CPUMeter to inherit from FieldMeterGraph, rather than
//       FieldMeter.
//   3.  Change the constructor call to use FieldMeterGraph(), rather than
//       FieldMeter().
//   4.  Make the meter call FieldMeterGraph::checkResources(),
//       to pick up graphNumCols resource.
//   5.  Make the checkResources () function in the meter set the 
//	 useGraph_ variable according to the, e.g., xosview*cpuGraph resource.

#ifdef HAVE_FSTREAM
#include <fstream>
#else
#include <fstream.h>
#endif
#include <math.h>		//  For fabs()
#include "general.h"
#include "fieldmeter.h"
#include "fieldmetergraph.h"
#include "xosview.h"

CVSID("$Id$");
CVSID_DOT_H(FIELDMETERGRAPH_H_CVSID);

template FieldMeterGraph<float>::FieldMeterGraph(XOSView *, int, const char *, const char *, int, int, int);
template FieldMeterGraph<float>::~FieldMeterGraph(void);
template void FieldMeterGraph<float>::drawfields(int);
template void FieldMeterGraph<float>::drawBar(int);
template void FieldMeterGraph<float>::checkResources(void);
template void FieldMeterGraph<float>::setNumCols(int);

template FieldMeterGraph<double>::FieldMeterGraph(XOSView *, int, const char *, const char *, int, int, int);
template FieldMeterGraph<double>::~FieldMeterGraph(void);
template void FieldMeterGraph<double>::drawfields(int);
template void FieldMeterGraph<double>::drawBar(int);
template void FieldMeterGraph<double>::checkResources(void);
template void FieldMeterGraph<double>::setNumCols(int);

template<typename F>
FieldMeterGraph<F>::FieldMeterGraph( XOSView *parent,
                int numfields, const char *title,
                const char *legend, int docaptions, int dolegends, 
  int dousedlegends )
: FieldMeterDecay<F> (parent, numfields, title, legend, docaptions, 
  dolegends, dousedlegends)
{

	useGraph_ = 0;
	heightfield_ = NULL;
	firstTimeDrawn_ = 1;
	
	// set number of columns to a reasonable default in case we can't
	// find the resource
	setNumCols( 100 );

}

template<typename F>
FieldMeterGraph<F>::~FieldMeterGraph( void )
{
	delete [] heightfield_;
}

template<typename F>
void FieldMeterGraph<F>::drawfields( int manditory )
{
	int i,j;

	if( !useGraph_ )
	{
		// Call FieldMeterDecay code if this meter should not be
		// drawn as a graph
		FieldMeterDecay<F>::drawfields( manditory );
		return;
	}

	if( this->total_ <= 0.0 )
		return;

	// allocate memory for height field graph storage
	// note: this is done here as it is not certain that both
	// numfields_ and graphNumCols_ are defined in the constructor
	if( heightfield_ == NULL )
	{
		if( this->numfields_ > 0 && graphNumCols_ > 0 )
		{
			heightfield_ = new float [this->numfields_*graphNumCols_];
	
			for( i = 0; i < graphNumCols_; i++ )
			{
				for( j = 0; j < this->numfields_; j++ )
				{
					if( j < this->numfields_-1 )
						heightfield_[i*this->numfields_+j] = 0.0;
					else
						heightfield_[i*this->numfields_+j] = 1.0;
				}
			}
		}
	}

	// check current position here and slide graph if necessary
	if( graphpos_ >= graphNumCols_ )
	{
		for( i = 0; i < graphNumCols_-1; i++ )
		{
			for( j = 0; j < this->numfields_; j++ )
			{
				heightfield_[i*this->numfields_+j] = heightfield_[(i+1)*this->numfields_+j];
			}
		}
		graphpos_ = graphNumCols_ - 1;
	}

	// get current values to be plotted
	for( i = 0; i < this->numfields_; i++ )
	{
		F a = this->fields_[i] / this->total_;
		if( a <= 0.0 )
			a = 0.0;
		if( a >= 1.0 )
			a = 1.0;
		heightfield_[graphpos_*this->numfields_+i] = a;
	}

	/*  For the first time, we need to draw everything, so
	 *  skip the optimized copyArea case.  Also, if we are
	 *  not fully visible, then the copy-area won't work
	 *  properly.  */
	if( !firstTimeDrawn_ && this->parent_->hasBeenExposedAtLeastOnce() && !this->parent_->isExposed() && this->parent_->isFullyVisible() )
	{
		// scroll area
		int col_width = this->width_/graphNumCols_;
		if( col_width < 1 )
		{
			col_width = 1;
		}

		int sx = this->x_ + col_width;
		int swidth = this->width_ - col_width;
		int sheight = this->height_ + 1;
		if( sx > this->x_ && swidth > 0 && sheight > 0 )
			this->parent_->copyArea( sx, this->y_, swidth, sheight, this->x_, this->y_ );
		drawBar( graphNumCols_ - 1 );
	} else {
		if (firstTimeDrawn_ &&
		    this->parent_->isAtLeastPartiallyVisible() &&
		    this->parent_->hasBeenExposedAtLeastOnce()) {
			XOSDEBUG("True exposure! %d\n", firstTimeDrawn_);
			firstTimeDrawn_ = 0;
		}
		else XOSDEBUG("Full draw:  isAtLeastPart %d, hasBeenExposed %d\n",
			this->parent_->isAtLeastPartiallyVisible(),
			this->parent_->hasBeenExposedAtLeastOnce());
		// need to draw entire graph on expose event
		for( i = 0; i < graphNumCols_; i++ ) {
			drawBar( i );
		}
	}

	graphpos_++;
    this->parent_->setStippleN(0);	//  Restore all-bits stipple.
    if ( this->dousedlegends_ )
    {
    	this->drawused( manditory );
    }
}

template<typename F>
void FieldMeterGraph<F>::drawBar( int i )
{
	int j;
	int y = this->y_ + this->height_;
	int x = this->x_ + i*this->width_/graphNumCols_;
	int barwidth = (this->x_ + (i+1)*this->width_/graphNumCols_)-x;

	if( barwidth>0 )
	{
		int barheight;
		for( j = 0 ; j < this->numfields_; j++ )
		{
			/*  Round up, by adding 0.5 before
		 	*  converting to an int.  */
			barheight = (int)((heightfield_[i*this->numfields_+j]*this->height_)+0.5);

			this->parent_->setForeground( this->colors_[j] );
  			this->parent_->setStippleN(j%4);

			if( barheight > (y-this->y_) )
  				barheight = (y-this->y_);

			// hack to ensure last field always reaches top of graph area
			if( j == this->numfields_-1 )
				barheight = (y-this->y_);

			y -= barheight;
			if( barheight>0 )
				this->parent_->drawFilledRectangle( x, y, barwidth, barheight );
		}
	}
}

template<typename F>
void FieldMeterGraph<F>::checkResources( void )
{
  FieldMeterDecay<F>::checkResources();

  const char *ptr = this->parent_->getResource( "graphNumCols" );
  if( ptr )
  {
    int i;
	if( sscanf( ptr, "%d", &i ) == 1 )
	{
		if( i>0 )
		{
			setNumCols( i );
		}
	}
  }
}

template<typename F>
void FieldMeterGraph<F>::setNumCols( int n )
{
	graphNumCols_ = n;
	graphpos_ = graphNumCols_-1;

	if( heightfield_ )
		delete [] heightfield_;
	heightfield_ = NULL;

}

